#!/bin/sh

## This pipeline runs the following steps:
## 
##  1- move the fasta files into job directories
##  2- run mafft on the fasta files 
##  3- run trimal to clean up the mafft alns 
##  4- convert resulting aln to phy
##  5- run raxml to create trees 

### USER SPECIFY PBS VARIABLES
export JOBS=50				# number of blast jobs to split the query into
QUEUE='standard'			# PBS queue
GROUP='group_list=hackettj'		# PBS group id
USER='hughesj@email.arizona.edu'	# PBS user id
JTYPE='jobtype=htc_only'		# PBS job type
NCPUS='select=1:ncpus=1:mem=2gb'	# PBS resource request
CPUT='cput=72:0:0' 				# PBS resource request
WT='walltime=72:0:0'			# PBS resource request
PPS='place=pack:shared'			# PBS resource request

### USER SPECIFY PHYLOGENETIC VARIABLES
export MINSEQ='4'  		# minimum sequences for tree building
export MAXSEQ='200'		# maximum sequences for tree building
export BLOSSUM='62'		# blossum matrix

### DIRECTORY INFO
export FINALDIR="/path/to/phylopipe"
export FADIR="$FINALDIR/query"
SCRIPTS="$FINALDIR/scripts"
export CWD=$PWD

##  1 - split the files equally between # JOB directories 
FIRST=`qsub -v CWD,FADIR,FINALDIR,JOBS -N 1splitjobs -q $QUEUE -W $GROUP -M $USER -l $JTYPE -l $NCPUS -l $CPUT -l $WT -o $CWD/pbsout -e $CWD/pbsout $SCRIPTS/run_split.sh`

##  2 - clean up files, remove illegal characters, remove identical sequenes, remove files if totalseqs<n
SECOND=`qsub -v CWD,FINALDIR,FADIR,MINSEQ -N 2clean -q $QUEUE -W $GROUP -M $USER -l $JTYPE -l $NCPUS -l $CPUT -l $WT -o $CWD/pbsout -e $CWD/pbsout -J 1-$JOBS $SCRIPTS/run_clean.sh`

##  3 - run mafft reorder to rank sequences in decreasing pairwise similarity to top sequence
THIRD=`qsub -v CWD,FINALDIR,FADIR -W depend=afterok:$SECOND -N 3reord -q $QUEUE -W $GROUP -M $USER -l $JTYPE -l $NCPUS -l $CPUT -l $WT -o $CWD/pbsout -e $CWD/pbsout -J 1-$JOBS $SCRIPTS/run_reord.sh`

##  4 - take the top N sequences in the fasta file
FOURTH=`qsub -v CWD,FINALDIR,FADIR,MAXSEQ -W depend=afterok:$THIRD -N 4head -q $QUEUE -W $GROUP -M $USER -l $JTYPE -l $NCPUS -l $CPUT -l $WT -o $CWD/pbsout -e $CWD/pbsout -J 1-$JOBS $SCRIPTS/run_head.sh` 

##  5 - run mafft to aln 
FIFTH=`qsub -v CWD,FINALDIR,FADIR,BLOSSUM -W depend=afterok:$FOURTH -N 5mafft -q $QUEUE -W $GROUP -M $USER -l $JTYPE -l $NCPUS -l $CPUT -l $WT -o $CWD/pbsout -e $CWD/pbsout -J 1-$JOBS $SCRIPTS/run_mafft.sh`

##  6 - run trimal to clean aln 
SIXTH=`qsub -v CWD,FINALDIR,FADIR -W depend=afterok:$FIFTH -N 6trimal -q $QUEUE -W $GROUP -M $USER -l $JTYPE -l $NCPUS -l $CPUT -l $WT -o $CWD/pbsout -e $CWD/pbsout -J 1-$JOBS $SCRIPTS/run_trimal.sh`

##  8 - run phylogeny software 
SEVENTH=`qsub -v CWD,FINALDIR,FADIR -W depend=afterok:$SIXTH -N 7fasttree -q $QUEUE -W $GROUP -M $USER -l $JTYPE -l $NCPUS -l $CPUT -l $WT -o $CWD/pbsout -e $CWD/pbsout -J 1-$JOBS $SCRIPTS/run_fasttree.sh`  

