#!/usr/local/bin/perl -w
 

################# 
#
# reap_trimming.pl
# 
# Perl script to trim 
# multi sequence fasta files for treebuilding
#
# Run: perl reap_trimming.pl [OPTIONS]
#
#################
# Use the correct version of bioperl
use lib '/uaopt/bioperl/1.6.1/lib/perl5/site_perl/5.8.8/';

use strict;
use File::Spec;
use Getopt::Std;
use Cwd;
use Bio::AlignIO;
use Data::Dumper;

use Time::HiRes qw ( time alarm sleep );


#=========== INPUTS  ==========
my $dir = getcwd;
my %opts; getopt('fodawcneg', \%opts );

my $usage = "\n\tUsage Error:
	Run: perl reap_trimming.pl [OPTIONS]

	OPTIONS =
	Files:
	-f: <FILENAME> IFILE: sequence alignment in fasta format (specify path if needed)
	-o: <FILENAME> OFILE: name for new output file in fasta format
	
	General:
	-d: <INTEGER> DSCORE: gap score (default = 0)
	-a: <INTEGER> AMIN: minimun amino acids (default = 1)
	
	For removing columns:
	-w: <INTEGER> WSIZE windowsize: size of sliding window for evaluation (default = 6)
	-c: <INTEGER> CSCORE SPmin: minimum PAM250 column score (default = 3)
				 *note: parameter is relative to the window size
	-n: <INTEGER> NMIN nongapmin: minimum percentage of non gaps (default = 40)
	
	For removing sequences:
	-e: <INTEGER> SSCORE min_seq_score: minimum PAM250 score for a sequence (default = -100)
	-g: <DECIMAL> GMAX maxgp: maximum proportion of gaps allowed (default = 0.7)
	\n\n";

# Files:
my ($IFILE, $IPATH);
if ($opts{f}) { 
	$IPATH = "$opts{f}";
	if (-e "$opts{f}") {
		#print "\nalignment file found...";
		my @filepath = File::Spec->splitpath($opts{f}); 
		$IFILE = $filepath[2];
	}
	else { die "\n\tError: Input file <$IPATH> not found.\n\tInput <f>$usage"; }
}
else { die "\n\tInput <f>$usage"; }

my ($OFILE);
if ($opts{o}) { 
	$OFILE = "$opts{o}";
	if (-e "$opts{o}") {
		#print "\nOutput file specified...";
		my @filepath = File::Spec->splitpath($opts{o}); 
	}
}
else { die "\n\tInput <o>$usage"; }

# General
my $DSCORE; #gap score
if ($opts{d}) { if ($opts{d} =~ m/^\d+$/) { 
	if (int($opts{d}) eq $opts{d}) { $DSCORE = "$opts{d}"; }} else { die "\n\tInput <d>$usage"; } }
else { $DSCORE = 0; }

my $AMIN; #min amino acids
if ($opts{a}) { if ($opts{a} =~ m/^\d+$/) { 
	if (int($opts{a}) eq $opts{a}) { $AMIN = "$opts{a}"; }} else { die "\n\tInput <a>$usage"; } }
else { $AMIN = 1; }

# Column removal 
my $WSIZE; #sliding window size
if ($opts{w}) { if ($opts{w} =~ m/^\d+$/) { 
	if (int($opts{w}) eq $opts{w}) { $WSIZE = "$opts{w}"; }} else { die "\n\tInput <w>$usage"; } }
else { $WSIZE = 6; }

my $CSCORE; #min PAM250 score for column
if ($opts{c}) { if ($opts{c} =~ m/^\d+$/) { 
	if (int($opts{c}) eq $opts{c}) { $CSCORE = "$opts{c}"; }} else { die "\n\tInput <c>$usage"; } }
else { $CSCORE = 3; }

my $NMIN; #min percentage of non gaps
if ($opts{n}) { if ($opts{n} =~ m/^\d+$/) { 
	if (int($opts{n}) eq $opts{n}) { $NMIN = "$opts{n}"; }} else { die "\n\tInput <n>$usage"; } }
else { $NMIN = 40; }

# Sequence removal 
my $SSCORE; #min PAM250 score for sequence
if ($opts{e}) { if ($opts{e} =~ m/^\d+$/) { 
	if (int($opts{e}) eq $opts{e}) { $SSCORE = "$opts{e}"; }} else { die "\n\tInput <e>$usage"; } }
else { $SSCORE = -100; }

my $GMAX; #max proportion of gaps
if ($opts{g}) { if ($opts{g} =~ m/^\d+[\/|\.]\d+$/) { $GMAX = "$opts{g}"; } else {die "\n\tInput <g>$usage";} }  
else { $GMAX = "0.7"; }


#=========== PAM250 matix  ==========
my $amino_acids_positions = 
                  {"A"=>0, "R"=>1, "N"=>2, "D"=>3, "C"=>4, "Q"=>5, "E"=>6, "G"=>7, "H"=>8, "I"=>9, "L"=>10, "K"=>11,
                   "M"=>12, "F"=>13, "P"=>14, "S"=>15, "T"=>16, "W"=>17, "Y"=>18, "V"=>19, "B"=>20, "Z"=>21,
                   "X"=>22, "-"=>23};

                                                                        
my @matrix = (
[2],
[-2, 6],
[ 0, 0, 2],
[ 0, -1, 2, 4],
[-2, -4, -4, -5, 12],
[ 0, 1, 1, 2, -5, 4],
[ 0, -1, 1, 3, -5, 2, 4],
[ 1, -3, 0, 1, -3, -1, 0, 5],
[-1, 2, 2, 1, -3, 3, 1, -2, 6],
[-1, -2, -2, -2, -2, -2, -2, -3, -2, 5],
[-2, -3, -3, -4, -6, -2, -3, -4, -2, 2, 6],
[-1, 3, 1, 0, -5, 1, 0, -2, 0, -2, -3, 5],
[-1, 0, -2, -3, -5, -1, -2, -3, -2, 2, 4, 0, 6],
[-4, -4, -4, -6, -4, -5, -5, -5, -2, 1, 2, -5, 0, 9],
[ 1, 0, -1, -1, -3, 0, -1, -1, 0, -2, -3, -1, -2, -5, 6],
[ 1, 0, 1, 0, 0, -1, 0, 1, -1, -1, -3, 0, -2, -3, 1, 2],
[ 1, -1, 0, 0, -2, -1, 0, 0, -1, 0, -2, 0, -1, -3, 0, 1, 3],
[-6, 2, -4, -7, -8, -5, -7, -7, -3, -5, -2, -3, -4, 0, -6, -2, -5, 17],
[-3, -4, -2, -4, 0, -4, -4, -5, 0, -1, -1, -4, -2, 7, -5, -3, -3, 0, 10],
[ 0, -2, -2, -2, -2, -2, -2, -1, -2, 4, 2, -2, 2, -1, -1, -1, 0, -6, -2, 4],
[ 0, -1, 2, 3, -4, 1, 2, 0, 1, -2, -3, 1, -2, -5, -1, 0, 0, -5, -3, -2, 2],
[ 0, 0, 1, 3, -5, 3, 3, -1, 2, -2, -3, 0, -2, -5, 0, 0, -1, -6, -4, -2, 2, 3],
[ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0],
[ 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0]
);


#========== REAP TRIMMING ==========
print "\n\nOriginal $IFILE...\n";

#Read in alignment file ***Can make infile format a command line variable later
my ($aln_array,$aln_ids,$alignment_length,$sequence_count) = &readAlignment ($IPATH, "fasta");
print "\tseq count: $sequence_count\n\talign length: $alignment_length\n";
	#if ($opt_verbose) {print "\nalignment ids\n" . join(",", @$aln_ids) . "\nseq count: $sequence_count\naln_len: $alignment_length\n";}


#Calculate SP Scores
my ($sp_array, $no_gap_percentage_array) = &calculateSP ($aln_array, "-");
	#if ($opt_verbose) { print "\nnon-gap percentiles:\n" . join(",", @$no_gap_percentage_array). "\n"; }


#Find quality windows column by column ***Retained column coordinates ARE inclusive.  
my ($rc,$rc_count) = &findRetainedColumns ($sp_array,$no_gap_percentage_array,$WSIZE,$CSCORE,$NMIN,$alignment_length,$sequence_count);
	#if($opt_verbose) { print "\nretained columns (array) BEFORE mergeBlocks:\n" . join(",", @$rc) . "\ncount\n$rc_count\n"; }
($rc,$rc_count) = &mergeBlocks($rc);
	#if($opt_verbose) { print "\nretained columns (array) AFTER mergeBlocks: \n" . join(",", @$rc). "\ncount: $rc_count\n"; }


#Find retained and excluded sequences
if ($rc_count > 0) { 

  #Retain sequences based on PAM250 score
  my ($rs, $es) = &findRetainedSequences ($aln_array, "-" ,$sequence_count,$rc_count,$GMAX,$rc); 
  my $rs_count = @$rs;
  my $es_count = @$es;
  #if($opt_verbose) { foreach my $index (@$rs) { print "$index: $aln_ids->[$index]\n"; } foreach my $index (@$es) { print "$index: $aln_ids->[$index]\n"; }}

  #what do we mean by misaligned sequences anyway?
  my ($mis_aligned, $ok_aligned) = &findMisalignedSequences ($rs, $rc, $rc_count, $aln_array, "-" ,$aln_ids);
  $rs_count = @$ok_aligned;
  
  #output reduced alignment
  &printReducedAlignment  ($OFILE, 'fasta' , $aln_array, $rc, $ok_aligned, $aln_ids);
  
  print "\nNew $OFILE...\n\tseq count: $rs_count\n";
  #foreach my $index (@$ok_aligned) { print "$aln_ids->[$index],"; }
  #print "\n";
  
  print "\talign length: $rc_count\n\n\n";
  #print join(",", @$rc) . "\n";
    
}else{ 
	print "No retained columns :(\n"; }


#========== MODULES by Stephanie Hartmann ==========

#This function receives a filename and alignment format
# it returns a reference to a two dimensional array representing 
# the alignment...rows=sequence columnes=alignments
sub readAlignment
{
  my ($filename, $format) = @_;

  my $aln_array = ();
  my $aln_ids = ();
  my $in  = Bio::AlignIO->new(-file => $filename ,
                             '-format' => "$format");
  while ( my $aln = $in->next_aln() ) 
  { 
    foreach my $seq ( $aln->each_seq() ) 
    {
      push @$aln_ids, $seq->id();
      my @arr = split('', $seq->seq);
      push @$aln_array, \@arr;
    }
  }
  my $sequence_count = @$aln_array;
  my $alignment_length   = @{$aln_array->[0]};
  
  #print "oriS: $sequence_count\noriC: $alignment_length\n";

  return ($aln_array,$aln_ids,$alignment_length,$sequence_count);
}

#look up the score of the two amino acids inside of the matrix.
sub pairScore
{
  my ($aa1, $aa2) = @_;     

  #The amino acids should be capitalized because they are capitalized inside of the hash as a key
  #  The value of the AA Key of the hash is the index in the matrix.
  $aa1 = uc($aa1); $aa2 = uc($aa2);
  if (( !(exists($amino_acids_positions->{$aa1}) )) || ( !( exists($amino_acids_positions->{$aa2}) )))
  { return $DSCORE; }
  my $pos1 = $amino_acids_positions->{$aa1};
  my $pos2 = $amino_acids_positions->{$aa2};

  #only half the matrix is full.  Switch the values if the column value(pos2) is greater
  # than the row value(pos1) 
  if ($pos2 > $pos1) { my $temp = $pos1; $pos1 = $pos2; $pos2 = $temp;}

  return $matrix[$pos1][$pos2];
}

  

# input is alignment file..its format and the gap character
# returns an SP Score for each column of the alignment and gap count for each column
#
#
sub calculateSP
{
  my ($aln_arr,$GAP_CHAR) = @_;

  my $n = @$aln_arr;        #sequence count
  my $m = @{@$aln_arr[0]};  #alignment length
  my @score = ();
  my @non_gap   = ();
  my $aa_hash_count = {};
  

  #loop through each column.
  for (my $k=0; $k <$m; $k++)                           #for each column
  {
    my $non_gap_count = 0;
    $score[$k] = 0;
    $non_gap[$k] = 0;

    #keep a count of each amino acid in the column.
    # store the count as a value in aa_bin_hash and the key is the amino acid
    my $aa_bin_hash = {};
    for (my $i = 0; $i<$n; $i++)
    {
      my $i_aa = $aln_arr->[$i][$k];
      if(exists ($aa_bin_hash->{$i_aa}))
      { $aa_bin_hash->{$i_aa} += 1; } 
      else { $aa_bin_hash->{$i_aa} = 1; } 

      if($i_aa ne $GAP_CHAR) 
      { $non_gap[$k]++; $non_gap_count++; } 
    }

    #make a two arrays of corresponding amino acid id and counts
    my @aa_bin_array = ();
    my @aa_bin_value_array = ();
    foreach my $key (keys (%$aa_bin_hash))
    { push @aa_bin_array,$key;
      push @aa_bin_value_array, $aa_bin_hash->{$key};
    }

    #now use more math than looping to calculate the SP scores
    $b = @aa_bin_array;
    $score[$k] = 0;
    for (my $i = 0; $i<$b; $i++)
    { 
      my $i_aa = $aa_bin_array[$i];
      my $i_aa_cnt = $aa_bin_value_array[$i];
      if($i_aa_cnt > 1)
      { $score[$k] += ($i_aa_cnt * ($i_aa_cnt-1)/2) * &pairScore($i_aa, $i_aa);     }

      for (my $j=($i+1); $j <$b; $j++)            
      { 
        my $j_aa = $aa_bin_array[$j];
        my $j_aa_cnt = $aa_bin_value_array[$j];
        $score[$k] += $j_aa_cnt * $i_aa_cnt * &pairScore($i_aa, $j_aa); 
      }
    }
  
    my $denominator = ($non_gap_count*($non_gap_count-1) );

    if($denominator == 0) { $score[$k] = 0; }
    else                  { $score[$k] /= $denominator; }
  }

  #now we loop one time throug the non_gap array...converting each non_gap count into
  #a percentage of non gap
  for (my $i=0; $i <$m; $i++) 
  {  $non_gap[$i] = ($non_gap[$i]/$n)*100; } 
  return (\@score, \@non_gap);
}



#$N = length of the entire initial alignment (must be greater than w)  - DEPENDS ON ALIGNMENT
#$S = number of sequences in the entire initial alignment              - DEPENDS ON ALIGNMENT
#
#$w    window size, sets the minimum length of a retained contiguous stretch of an alignment
#$m=6  minimal sum of SP scores (not including gaps)
#$s=60 minimal number (percentage?) of sequences present within a column
#$t    optional parameter: average number of gap openings within a window, among sequences
#$g    maximal proportion of gaps allowed within a sequence in the retained columns
#

sub findRetainedColumns 
{
  my ($column_SPscore, $column_nongaps,$w,$m,$s,$N,$S) = @_;
  my $i = 0;                                  # start with the first column
  my ($x) = (0);                              # initialize variables for window sums
  my $in_block = 0;
  my @rc = ();
  my $retained_count = 0;


  my ($left_border,$right_border) = (0,0);
  
  my $no_accept_count = 0;
  my @gappy = ();

  for (my $j = $i; $j<=$i+$w-1 ; $j++ ) 
  {                                            # for the very first window           
    $x += $column_SPscore->[$j];               # calculate SP scores for the first window
    if ($column_nongaps->[$j] < $s) 
    {                                          # deal with gappiness here
      push (@gappy, 0);
      $no_accept_count ++;
    } 
    else 
    { push (@gappy, 1); }
  }    

  do {
    if ( ($x > $m)  &&  ($no_accept_count == 0) ) 
    {                                             # if values for SP scores and nongaps are 
      if (! $in_block)                            # above the pre-determined threshold
      {                                           # and if we're not within a block
        $in_block = 1;                            # change so that we're in a block and 
        $left_border =  $i;
        $right_border = $i+$w-1;
      } 
      else 
      { $right_border = $i+$w-1; }
    } 
    else                                          # if value  for SP scores and nongaps are NOT 
    {                                             # above the threshold
      if ($in_block)                              # and if we're in a block
      {
         push @rc, ($left_border, $right_border); 
         $retained_count += $right_border-$left_border +1;
         $in_block = 0;                           # change so that we're not in a block any more 

      }
    }
                                                  # calculate sums for subsequent windows differently
    $i++; 
    if($i+$w-1 < $N)  #escape loop if looking beyond the end of alignment
    {
         $x = ($x - ($column_SPscore->[$i-1]) + ($column_SPscore->[$i+$w-1]) );
         my $next_one_out = shift @gappy;
         if ($next_one_out == 0)
         { $no_accept_count --; }
         if ($column_nongaps->[$i+$w-1] < $s) 
         {  
           push (@gappy, 0);
           $no_accept_count ++;
         } 
         else { push (@gappy, 1); }    
         
         #if ($opt_verbose) {print "$i: $x, $left_border, $right_border, gappy: @gappy, NAC: $no_accept_count \n";}
         #my $d = <STDIN>;    
    }                     # DEBUG FUNCTION
  } until ($i>$N-$w);

  #out of loop..still in block and no @rc elements?  sounds like one long complete block to me.
  if ($in_block)                             # and if we're in a block
  { push @rc, ($left_border, $right_border); 
    $retained_count += $right_border-$left_border +1;
    $in_block = 0;                           # change so that we're not in a block any more 

  }
  
  return (\@rc,$retained_count);

}






#$aln_array = 2-d array of the alignment
#$gap_char = the gap character in the alignment
#$sequence_count = number of sequences
#$n = number of retained sequences
#$g = the maximal proportion of gaps allowed within a sequence in the retained columns
#$rc = array of retained columns
sub findRetainedSequences 
{
  my ($aln_array, $gap_char,$sequence_count,$n,$g, $rc) = @_;
  my @retained_sequences = ();
  my @excluded_sequences = ();
  my $number_of_blocks = @$rc;                                 # calculate the number of elements in @rc
  my $i=0;
  for ($i=0; $i < $sequence_count; $i++) 
  {                                                            # loop through each sequence of @alignment
                                                               # loop through each retained column block
    my $j = 0;                        
    my $z=0;                                                   #z is a gap count
    my $y=0;                                                   #y is a amino acid count
    next if ($number_of_blocks == 0);
    while($j < $number_of_blocks) 
    {                                                          # as long as there are elements in @rc
      my $left_index = $rc->[$j];                              # get the left index 
      my $right_index = $rc->[$j+1];                           # and get the right index
      while ($left_index <= $right_index)  
      {                                                        # while we're within a retained column block
        if ($aln_array->[$i][$left_index] eq $gap_char) 
        { $z++; }                                              # add 1 for each gap residue to variable 
        else
        { $y++; }                                              # add 1 for each nongap residue to variable 

        $left_index++;                                         # to move along the retained column block
      }
      $j += 2;                                                 # to move to the next LEFT INDEX within @rc
    }	

    #if ($opt_verbose){ print "($z/$n < $g)  ($y > $AMIN)\n"; }
    if (((($z/$n) < $g)) && ($y > $AMIN))
    {                                                          # if the seq has a smaller proportion of 
      push (@retained_sequences, $i);                          # gaps than the threshold "g", add it to the 
    }                                                          # list of retained sequences
    else 
    {
      push (@excluded_sequences, $i);
    }
  }


  #print "sequences retained (gaps):    " . @retained_sequences . "\nsequences excluded (gaps):    " . @excluded_sequences . "\n";
  #print "sequences excluded (gaps):    " . @excluded_sequences . "\n";

  return (\@retained_sequences, \@excluded_sequences);         # return an arrays of retained and excluded sequences 
}



sub findMisalignedSequences           
{
  my ($retained_sequences,$rc, $col_count,$aln_array,$gap_char,$seq_names) = @_;
  my @misaligned_sequences = ();                                # array with the misaligned sequences
  my @ok_aligned_sequences = ();
  my $number_of_blocks = @$rc;
  my $number_of_rs = @$retained_sequences;
  my $i = 0; 


  foreach my $i (@$retained_sequences)                         # loop through all retained sequences
  {                                        
    my $score_columns = 0;                                     # average column-score of a sequence with other sequs
    my $score_total = 0;                                       # average total score of a sequence within the alignment
    my $j = 0;     
    my $aa_counter = 0;                   
    while($j < $number_of_blocks)                              # loop through all retained columns
    {                                                          # as long as there are elements in @rc
      my $left_index = $rc->[$j];                              # get the left index 
      my $right_index = $rc->[$j+1];                           # and get the right index
      while ($left_index <= $right_index)  
      {                                                        # while we're within a retained column block
 
        # sum pairwise scores of current sequence and all other retained sequences within this column
        if($aln_array->[$i][$left_index] ne $gap_char)
        {  
          $score_columns += &colScoreOneVsAll ($retained_sequences, $i,$left_index,$aln_array,$gap_char);
          $aa_counter++; 
        }                                             

        $left_index++;                                         # to move along the retained column block
      }
      $j += 2;                                                 # to move to the next LEFT INDEX within @rc
    }	


    $score_columns /= $number_of_rs;
    
    # sum pairwise scores for all columns


    #$score_total = ($score_columns / $col_count);             # divide by total number of columns: eliminates
                                                               # short sequences, even if they are well aligned!

    $score_total = ($score_columns / $aa_counter);             # divide by number of columns for that sequence: better
    
    #print "\n\n $i:  score total ($score_total) = score_columns ($score_columns)  / aa_counter ($aa_counter)\n\n";
    
    

    if    ($score_total <= $SSCORE)    {push (@misaligned_sequences, $i); } 
    else                            {push (@ok_aligned_sequences, $i);}
   
    #if ($opt_verbose) { print $i. ": " . $aa_counter . " AAs  " . $seq_names->[$i] . ": score_total = " . $score_total . "\n"; }

  }
  #print "sequences misaligned (score): " . @misaligned_sequences . "\nsequences ok aligned (score): " . @ok_aligned_sequences . "\n";
  #print "sequences misaligned (score): " . @misaligned_sequences . "\n";

  #if ($opt_verbose) { print "\nmisaligned sequences:\n" . join(",", @misaligned_sequences) . "\n\n"; print "ok_aligned sequences:\n" . join(",", @ok_aligned_sequences) . "\n\n"; }
  
  return (\@misaligned_sequences, \@ok_aligned_sequences);     
}


sub returnExcludedColumns 
{
  my ($rc, $alignment_length) = @_;

  my @excluded_columns = ();
  #my $excluded_columns_count;
  my $borders;

  push (@excluded_columns, 0);

  my $alt = -1;

  foreach $borders (@$rc) 
  { 
    push (@excluded_columns, ($borders + $alt) );
    if    ($alt == -1) { $alt =  1; } 
    elsif ($alt ==  1) { $alt = -1; } 
  }
  push (@excluded_columns, ($alignment_length-1));
  
  return (\@excluded_columns);
}



#
# mergeBlocks -> receives an array of blocks and returns the merger of it.
#
#  For this function to properly merge an array of blocks together,
#  it assumes a few things about the input blocks.  If these assumptions are 
#  not met, then the results will be unreliable and inaccurate.
#  1.  the array is full of pairs such that an even indexed value is
#      a left-border(inclusive) and it's adjacent odd indexed value on the right
#      (thats left-border-index +1) is the right hand border.  One pair
#      represents a block.  For instance an array of the following values:
#            (10,12, 16,20, a,b) represent 3 blocks.  one from 10-12, another
#                 from 16-20 and finally a general block from a-b.
#  2.  Given two consecutive blocks (bl1, br1, bl2,br2), They are allowed to
#      overlap, but br2 must be greater than br1.  In a general form (bri+1 must be
#      > bri) where i is the index
#     Here is an example of acceptable blocks and one unacceptable block:
#     block: 0   ---------      (yes)
#            1     ---------    (yes)
#            2         -------  (yes)
#            3       -------    (no)
#     block 3 as does not satisfy the condition defined above (bri+1 must be > bri) 
#          
#
sub mergeBlocks
{
  my ($block_array) = @_;

  my $new_array = [];  
  
  my $left_border  = shift @$block_array;
  my $right_border = shift @$block_array;

  my $col_count = 0; 
  my ($new_left,$new_right) = (0,0);

  while (@$block_array)
  {
    $new_left  = shift @$block_array;
    $new_right = shift @$block_array || die "array has odd number of elements...should be even";
    if($new_left <= $right_border+1)
    {  $right_border = $new_right; } 
    else
    {  push @$new_array, ($left_border,$right_border);
       $col_count += $right_border - $left_border +1;
       $left_border = $new_left;
       $right_border = $new_right;
    }
    if (! (@$block_array))
    {  push @$new_array, ($left_border,$right_border);
       $col_count += $right_border - $left_border +1;

    }
  }

  #if the retained columns is represented by ONE block, we need to push 
  # that block back on before returning the array.
  if((! @$new_array) && ($left_border ne ""))
  { push @$new_array, ($left_border,$right_border);
    $col_count += $right_border - $left_border +1;
  }
  return ($new_array,$col_count);
}

#given an array of block bounds...return the count
sub countRetainedColumns
{
  my ($blocks_array)  = @_;

  my $cnt = @$blocks_array;
  
  my $i=0;
  my $col_count = 0;
  while($i <$cnt)
  { $col_count += $blocks_array->[$i+1] - $blocks_array->[$i] + 1;  
    $i +=2;
  }
  return $col_count;
}



# convert array indices into alignment indices
sub offsetBlockArray
{
  my ($block_array, $offset) = @_;

  my $new_block_array = ();  

  foreach my $element (@$block_array) 
  { push @$new_block_array, ($element + $offset); }
  
  return $new_block_array;
}




#a file name
#format type
#a 2-d array
#retained column index list (corresponding to 2-d array)
#retained sequence index list (corresponding to 2-d array)
#retained sequence names list (corresponding to retained sequence index list)
# This input is enough to create an alignment file using
#AlignIO BioPerl Module.
sub printReducedAlignment
{
  my ($outfile, $format, $aln_arr, $rc, $rs, $aln_ids)  = @_;

  my $out_aln = Bio::AlignIO->new(-file => ">$outfile" ,
                                 '-format' => "$format");

  my $aln = Bio::SimpleAlign->new();

  my $rc_count = @$rc;
  
  foreach my $s_index(@$rs)
  { my $i = 0;
    my $cur_seq = "";
    my $seq_id = $aln_ids->[$s_index];
    
    while ($i < $rc_count)
    { #grab the left and right hand borders
      my $lb = $rc->[$i];
      my $rb = $rc->[$i+1];

      #append the good sequence onto $cur_seq
      while($lb <= $rb)
      { $cur_seq .= $aln_array->[$s_index][$lb++]; }
      $i += 2;

    }
    my $seq = new Bio::LocatableSeq(-seq => "$cur_seq",
                                    -id  => "$seq_id",
                                   );
    $aln->add_seq($seq);
  }

  $out_aln->write_aln($aln);
}


# average of sum pairwise scores of current sequence and all other retained sequences within this column
sub colScoreOneVsAll 
{
  my ($retained_sequences, $cur_seq,$cur_col,$aln_array,$gap_char) = @_;

  my $pair_score = 0;
  foreach my $i (@$retained_sequences)
  {
    next if ($i == $cur_seq);
    next if ($aln_array->[$i][$cur_col] eq $gap_char);
    $pair_score += &pairScore ($aln_array->[$cur_seq][$cur_col], $aln_array->[$i][$cur_col]);     
  }
  return $pair_score;

}

